@@ -1,15 +1,13 @@
 # $Id: dpkt.py 43 2007-08-02 22:42:59Z jon.oberheide $
 # -*- coding: utf-8 -*-
 """Simple packet creation and parsing."""
-from __future__ import absolute_import 
+from __future__ import absolute_import
 
 import copy
-import socket
 import struct
-import array
 from functools import partial
 
-from .compat import compat_ord, compat_izip, iteritems
+from .compat import compat_ord, compat_izip, iteritems, ntole
 
 
 class Error(Exception):
@@ -45,10 +43,10 @@ class _MetaPacket(type):
 
 
 class Packet(_MetaPacket("Temp", (object,), {})):
-    """Base packet class, with metaclass magic to generate members from self.__hdr__.
+    r"""Base packet class, with metaclass magic to generate members from self.__hdr__.
 
     Attributes:
-        __hdr__: Packet header should be defined as a list of 
+        __hdr__: Packet header should be defined as a list of
                  (name, structfmt, default) tuples.
         __byte_order__: Byte order, can be set to override the default ('>')
 
@@ -121,32 +119,32 @@ class Packet(_MetaPacket("Temp", (object,), {})):
         # 3. dynamically added fields from self.__dict__, unless they are _private
         # 4. self.data when it's present
 
-        l = []
+        l_ = []
         # maintain order of fields as defined in __hdr__
         for field_name, _, _ in getattr(self, '__hdr__', []):
             field_value = getattr(self, field_name)
             if field_value != self.__hdr_defaults__[field_name]:
                 if field_name[0] != '_':
-                    l.append('%s=%r' % (field_name, field_value))  # (1)
+                    l_.append('%s=%r' % (field_name, field_value))  # (1)
                 else:
                     # interpret _private fields as name of properties joined by underscores
                     for prop_name in field_name.split('_'):        # (2)
                         if isinstance(getattr(self.__class__, prop_name, None), property):
-                            l.append('%s=%r' % (prop_name, getattr(self, prop_name)))
+                            l_.append('%s=%r' % (prop_name, getattr(self, prop_name)))
         # (3)
-        l.extend(
+        l_.extend(
             ['%s=%r' % (attr_name, attr_value)
              for attr_name, attr_value in iteritems(self.__dict__)
-             if attr_name[0] != '_'                   # exclude _private attributes
-             and attr_name != self.data.__class__.__name__.lower()])  # exclude fields like ip.udp
+             if attr_name[0] != '_' and                   # exclude _private attributes
+                attr_name != self.data.__class__.__name__.lower()])  # exclude fields like ip.udp
         # (4)
         if self.data:
-            l.append('data=%r' % self.data)
-        return '%s(%s)' % (self.__class__.__name__, ', '.join(l))
+            l_.append('data=%r' % self.data)
+        return '%s(%s)' % (self.__class__.__name__, ', '.join(l_))
 
     def __str__(self):
         return str(self.__bytes__())
-    
+
     def __bytes__(self):
         return self.pack_hdr() + bytes(self.data)
 
@@ -156,7 +154,7 @@ class Packet(_MetaPacket("Temp", (object,), {})):
             return self._pack_hdr(
                 *[getattr(self, k) for k in self.__hdr_fields__]
             )
-        except struct.error:
+        except (TypeError, struct.error):
             vals = []
             for k in self.__hdr_fields__:
                 v = getattr(self, k)
@@ -180,8 +178,13 @@ class Packet(_MetaPacket("Temp", (object,), {})):
             setattr(self, k, v)
         self.data = buf[self.__hdr_len__:]
 
+
 # XXX - ''.join([(len(`chr(x)`)==3) and chr(x) or '.' for x in range(256)])
-__vis_filter = b'................................ !"#$%&\'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[.]^_`abcdefghijklmnopqrstuvwxyz{|}~.................................................................................................................................'
+__vis_filter = (
+    b'................................ !"#$%&\'()*+,-./0123456789:;<=>?'
+    b'@ABCDEFGHIJKLMNOPQRSTUVWXYZ[.]^_`abcdefghijklmnopqrstuvwxyz{|}~.'
+    b'................................................................'
+    b'................................................................')
 
 
 def hexdump(buf, length=16):
@@ -200,16 +203,17 @@ def hexdump(buf, length=16):
 def in_cksum_add(s, buf):
     n = len(buf)
     cnt = (n // 2) * 2
-    a = array.array('H', buf[:cnt])
+    a = struct.unpack('<{}H'.format(n // 2), buf[:cnt])  # unpack as little endian words
+    res = s + sum(a)
     if cnt != n:
-        a.append(compat_ord(buf[-1]))
-    return s + sum(a)
+        res += compat_ord(buf[-1])
+    return res
 
 
 def in_cksum_done(s):
     s = (s >> 16) + (s & 0xffff)
     s += (s >> 16)
-    return socket.ntohs(~s & 0xffff)
+    return ntole(~s & 0xffff)
 
 
 def in_cksum(buf):
@@ -222,5 +226,90 @@ def test_utils():
     __hd = '  0000:  00 01 02 03 04 05 06 07 08 09 0a 0b 0c 0d 0e     ...............'
     h = hexdump(__buf)
     assert (h == __hd)
+    assert in_cksum_add(0, __buf) == 12600  # endianness
     c = in_cksum(__buf)
     assert (c == 51150)
+
+
+def test_getitem():
+    """create a Packet subclass and access its properties"""
+    import pytest
+
+    class Foo(Packet):
+        __hdr__ = (
+            ('foo', 'I', 1),
+            ('bar', 'H', 2),
+        )
+
+    foo = Foo(foo=2, bar=3)
+    assert foo.foo == 2
+    assert foo['foo'] == 2
+    assert foo.bar == 3
+    assert foo['bar'] == 3
+
+    with pytest.raises(KeyError):
+        foo['grill']
+
+
+def test_pack_hdr_overflow():
+    """Try to fit too much data into struct packing"""
+    import pytest
+
+    class Foo(Packet):
+        __hdr__ = (
+            ('foo', 'I', 1),
+            ('bar', 'I', (1, 2)),
+        )
+
+    foo = Foo(foo=2**32)
+    with pytest.raises(PackError):
+        bytes(foo)
+
+
+def test_pack_hdr_tuple():
+    """Test the unpacking of a tuple for a single format string"""
+    class Foo(Packet):
+        __hdr__ = (
+            ('bar', 'II', (1, 2)),
+        )
+
+    foo = Foo()
+    b = bytes(foo)
+    assert b == b'\x00\x00\x00\x01\x00\x00\x00\x02'
+
+
+def test_unpacking_failure():
+    # during dynamic-sized unpacking in the subclass there may be struct.errors raised,
+    # but if the header has unpacked correctly, a different error is raised by the superclass
+    import pytest
+
+    class TestPacket(Packet):
+        __hdr__ = (('test', 'B', 0),)
+
+        def unpack(self, buf):
+            Packet.unpack(self, buf)
+            self.attribute = struct.unpack('B', buf[1:])
+
+    with pytest.raises(UnpackError, match="invalid TestPacket: "):
+        TestPacket(b'\x00')  # header will unpack successfully
+
+
+def test_repr():
+    class TestPacket(Packet):
+        __hdr__ = (('_a_b', 'B', 0),)
+
+        @property
+        def a(self):
+            return self._a_b >> 4
+
+        @property
+        def b(self):
+            return self._a_b & 0xf
+
+    # default values so no output
+    test_packet = TestPacket()
+    assert repr(test_packet) == "TestPacket()"
+
+    # non-default values
+    test_packet = TestPacket(b'\x12')
+    assert repr(test_packet) == "TestPacket(a=1, b=2)"
