@@ -120,10 +120,8 @@ class IP6(dpkt.Packet):
             s = struct.pack('>16s16sxBH', self.src, self.dst, self.p, len(p))
             s = dpkt.in_cksum_add(0, s)
             s = dpkt.in_cksum_add(s, p)
-            try:
-                self.data.sum = dpkt.in_cksum_done(s)
-            except AttributeError:
-                pass
+            self.data.sum = dpkt.in_cksum_done(s)
+
         return self.pack_hdr() + hdr_str + bytes(self.data)
 
     @classmethod
@@ -286,6 +284,7 @@ class IP6ESPHeader(IP6ExtensionHeader):
         dpkt.Packet.unpack(self, buf)
         self.length = self.__hdr_len__ + len(self.data)
 
+
 EXT_HDRS_CLS = {ip.IP_PROTO_HOPOPTS: IP6HopOptsHeader,
                 ip.IP_PROTO_ROUTING: IP6RoutingHeader,
                 ip.IP_PROTO_FRAGMENT: IP6FragmentHeader,
@@ -453,13 +452,14 @@ def test_ip6_gen_tcp_ack():
     _p, exthdrs = ipp.headers_str()
     ipp.plen = len(exthdrs) + len(ipp.data)
 
-    _b = bytes(ipp)
+    assert bytes(ipp)
 
     assert ipp.p == ip.IP_PROTO_TCP
     assert ipp.nxt == ip.IP_PROTO_HOPOPTS
 
 
 def test_ip6_opts():
+    import pytest
     # https://github.com/kbandla/dpkt/issues/477
     s = (b'\x52\x54\x00\xf3\x83\x6f\x52\x54\x00\x86\x33\xd9\x86\xdd\x60\x00\x00\x00\x05\x08\x3a\xff'
          b'\xfd\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\xfd\x00\x00\x00\x00\x00'
@@ -478,21 +478,59 @@ def test_ip6_opts():
     assert Ethernet(s).ip6.icmp6
     assert Ethernet(s).ip6.icmp6.data
 
-    try:
+    with pytest.raises(dpkt.NeedData):
         IP6(Ethernet(s).ip6.icmp6.data)  # should raise NeedData
-    except dpkt.NeedData:
+
+    from binascii import unhexlify
+    buf_ip6_opts = unhexlify(
+        '00'  # nxt
+        '00'  # len
+
+        '000000000000'  # only padding
+    )
+    ip6opt = IP6OptsHeader(buf_ip6_opts)
+    assert ip6opt.options == []
+    assert ip6opt.data == b'\x00' * 6
+
+
+def test_ip6_routing_properties():
+    ip6rh = IP6RoutingHeader()
+    assert ip6rh.sl_bits == 0
+    ip6rh.sl_bits = 1024
+    assert ip6rh.sl_bits == 1024
+
+
+def test_ip6_fragment_properties():
+    ip6fh = IP6FragmentHeader()
+    assert ip6fh.frag_off == 0
+    ip6fh.frag_off = 12345
+    assert ip6fh.frag_off == 12345
+
+    assert ip6fh.m_flag == 0
+    ip6fh.m_flag = 1
+    assert ip6fh.m_flag == 1
+
+
+def test_ip6_properties():
+    ip6 = IP6()
+
+    assert ip6.v == 6
+    ip6.v = 10
+    assert ip6.v == 10
+
+    assert ip6.fc == 0
+    ip6.fc = 5
+    assert ip6.fc == 5
+
+    assert ip6.flow == 0
+    ip6.flow = 4
+    assert ip6.flow == 4
+
+
+def test_proto_accessors():
+    class Proto:
         pass
 
-
-if __name__ == '__main__':
-    test_ipg()
-    test_ip6_routing_header()
-    test_ip6_fragment_header()
-    test_ip6_options_header()
-    test_ip6_ah_header()
-    test_ip6_esp_header()
-    test_ip6_extension_headers()
-    test_ip6_all_extension_headers()
-    test_ip6_gen_tcp_ack()
-    test_ip6_opts()
-    print('Tests Successful...')
+    assert 'PROTO' not in IP6._protosw
+    IP6.set_proto('PROTO', Proto)
+    assert IP6.get_proto('PROTO') == Proto
