@@ -149,7 +149,7 @@ class PktHdr(dpkt.Packet):
     Attributes:
         __hdr__: Header fields of pcap header.
         TODO.
-    """
+   """
     __hdr__ = (
         ('tv_sec', 'I', 0),
         ('tv_usec', 'I', 0),
@@ -170,7 +170,7 @@ class FileHdr(dpkt.Packet):
     Attributes:
         __hdr__: Header fields of pcap file header.
         TODO.
-    """
+   """
 
     __hdr__ = (
         ('magic', 'I', TCPDUMP_MAGIC),
@@ -195,7 +195,7 @@ class Writer(object):
     Attributes:
         __hdr__: Header fields of simple pcap dumpfile writer.
         TODO.
-    """
+   """
 
     __le = sys.byteorder == 'little'
 
@@ -221,7 +221,7 @@ class Writer(object):
         Args:
             pkt: `bytes` will be called on this and written to file.
             ts (float): Timestamp in seconds. Defaults to current time.
-        """
+       """
         if ts is None:
             ts = time.time()
 
@@ -233,7 +233,7 @@ class Writer(object):
         Args:
             pkt (bytes): Some `bytes` to write to the file
             ts (float): Timestamp in seconds
-        """
+       """
         n = len(pkt)
         sec = int(ts)
         usec = intround(ts % 1 * self._precision_multiplier)
@@ -248,7 +248,7 @@ class Writer(object):
 
         Args:
             pkts: iterable containing (ts, pkt)
-        """
+       """
         fd = self.__f
         pack_hdr = self._pack_hdr
         precision_multiplier = self._precision_multiplier
@@ -272,7 +272,7 @@ class Reader(object):
     Attributes:
         __hdr__: Header fields of simple pypcap-compatible pcap file reader.
         TODO.
-    """
+   """
 
     def __init__(self, fileobj):
         self.name = getattr(fileobj, 'name', '<%s>' % fileobj.__class__.__name__)
@@ -305,7 +305,7 @@ class Reader(object):
         return self.__fh.linktype
 
     def setfilter(self, value, optimize=1):
-        return NotImplementedError
+        raise NotImplementedError
 
     def readpkts(self):
         return list(self)
@@ -325,7 +325,7 @@ class Reader(object):
                     or 0 to process all packets until EOF
         callback -- function with (timestamp, pkt, *args) prototype
         *args    -- optional arguments passed to callback on execution
-        """
+       """
         processed = 0
         if cnt > 0:
             for _ in range(cnt):
@@ -354,6 +354,39 @@ class Reader(object):
             yield (hdr.tv_sec + (hdr.tv_usec / self._divisor), buf)
 
 
+################################################################################
+#                                    TESTS                                     #
+################################################################################
+class TryExceptException:
+    def __init__(self, exception_type, msg=''):
+        self.exception_type = exception_type
+        self.msg = msg
+
+    def __call__(self, f, *args, **kwargs):
+        def wrapper(*args, **kwargs):
+            try:
+                f()
+            except self.exception_type as e:
+                if self.msg:
+                    assert str(e) == self.msg
+            else:
+                raise Exception("There should have been an Exception raised")
+        return wrapper
+
+
+@TryExceptException(Exception, msg='There should have been an Exception raised')
+def test_TryExceptException():
+    """Check that we can catch a function which does not throw an exception when it is supposed to"""
+    @TryExceptException(NotImplementedError)
+    def fun():
+        pass
+
+    try:
+        fun()
+    except Exception as e:
+        raise e
+
+
 def test_pcap_endian():
     be = b'\xa1\xb2\xc3\xd4\x00\x02\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x60\x00\x00\x00\x01'
     le = b'\xd4\xc3\xb2\xa1\x02\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x60\x00\x00\x00\x01\x00\x00\x00'
@@ -362,14 +395,20 @@ def test_pcap_endian():
     assert (befh.linktype == lefh.linktype)
 
 
-def test_reader():
-    data = (  # full libpcap file with one packet
+class TestData():
+    pcap = (  # full libpcap file with one packet
         b'\xd4\xc3\xb2\xa1\x02\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\xff\xff\x00\x00\x01\x00\x00\x00'
         b'\xb2\x67\x4a\x42\xae\x91\x07\x00\x46\x00\x00\x00\x46\x00\x00\x00\x00\xc0\x9f\x32\x41\x8c\x00\xe0'
         b'\x18\xb1\x0c\xad\x08\x00\x45\x00\x00\x38\x00\x00\x40\x00\x40\x11\x65\x47\xc0\xa8\xaa\x08\xc0\xa8'
         b'\xaa\x14\x80\x1b\x00\x35\x00\x24\x85\xed'
     )
 
+
+def test_reader():
+    import pytest
+
+    data = TestData().pcap
+
     # --- BytesIO tests ---
     from .compat import BytesIO
 
@@ -379,6 +418,10 @@ def test_reader():
     assert reader.name == '<BytesIO>'
     _, buf1 = next(iter(reader))
     assert buf1 == data[FileHdr.__hdr_len__ + PktHdr.__hdr_len__:]
+    assert reader.datalink() == 1
+
+    with pytest.raises(NotImplementedError):
+        reader.setfilter(1, 2)
 
     # --- dispatch() tests ---
 
@@ -398,6 +441,68 @@ def test_reader():
     assert reader.dispatch(1, lambda ts, pkt: None) == 1
     assert reader.dispatch(1, lambda ts, pkt: None) == 0
 
+    # test loop() over all packets
+    fobj.seek(0)
+    reader = Reader(fobj)
+
+    class Count:
+        counter = 0
+
+        @classmethod
+        def inc(cls):
+            cls.counter += 1
+
+    reader.loop(lambda ts, pkt: Count.inc())
+    assert Count.counter == 1
+
+
+def test_reader_dloff():
+    from binascii import unhexlify
+    buf_filehdr = unhexlify(
+        'a1b2c3d4'    # TCPDUMP_MAGIC
+        '0001'        # v_major
+        '0002'        # v_minor
+        '00000000'    # thiszone
+        '00000000'    # sigfigs
+        '00000100'    # snaplen
+        '00000023'    # linktype (not known)
+    )
+
+    buf_pkthdr = unhexlify(
+        '00000003'  # tv_sec
+        '00000005'  # tv_usec
+        '00000004'  # caplen
+        '00000004'  # len
+    )
+
+    from .compat import BytesIO
+    fobj = BytesIO(buf_filehdr + buf_pkthdr + b'\x11'*4)
+    reader = Reader(fobj)
+
+    # confirm that if the linktype is unknown, it defaults to 0
+    assert reader.dloff == 0
+
+    assert next(reader) == (3.000005, b'\x11'*4)
+
+
+@TryExceptException(ValueError, msg="invalid tcpdump header")
+def test_reader_badheader():
+    from .compat import BytesIO
+    fobj = BytesIO(b'\x00' * 24)
+    _ = Reader(fobj)
+
+
+def test_reader_fd():
+    data = TestData().pcap
+
+    import tempfile
+    with tempfile.TemporaryFile() as fd:
+        fd.write(data)
+        fd.seek(0)
+        reader = Reader(fd)
+        assert reader.fd == fd.fileno()
+        assert reader.fileno() == fd.fileno()
+
 
 class WriterTestWrap:
     """
@@ -405,7 +510,8 @@ class WriterTestWrap:
 
     The test will be provided with a writer object, which it shoud write some pkts to.
 
-    After the test has run, the BytesIO object will be passed to a Reader, which will compare each pkt to the return value of the test.
+    After the test has run, the BytesIO object will be passed to a Reader,
+    which will compare each pkt to the return value of the test.
     """
     def __init__(self, *args, **kwargs):
         self.args = args
@@ -425,62 +531,72 @@ class WriterTestWrap:
                 fobj.seek(0)
 
                 assert pkts
-                for (ts_out, pkt_out), (ts_in, pkt_in) in zip(pkts, iter(Reader(fobj))):
+                for (ts_out, pkt_out), (ts_in, pkt_in) in zip(pkts, Reader(fobj).readpkts()):
                     assert ts_out == ts_in
                     assert pkt_out == pkt_in
-                writer.close()
+
+                # 'noqa' for flake8 to ignore these since writer was injected into globals
+                writer.close()  # noqa
                 Writer._Writer__le = _sysle
         return wrapper
 
+
 @WriterTestWrap()
 def test_writer_precision_normal():
     ts, pkt = 1454725786.526401, b'foo'
-    writer.writepkt(pkt, ts=ts)
+    writer.writepkt(pkt, ts=ts)  # noqa
     return [(ts, pkt)]
 
+
 @WriterTestWrap(writer={'nano': True})
 def test_writer_precision_nano():
     ts, pkt = Decimal('1454725786.010203045'), b'foo'
-    writer.writepkt(pkt, ts=ts)
+    writer.writepkt(pkt, ts=ts)  # noqa
     return [(ts, pkt)]
 
+
 @WriterTestWrap(writer={'nano': False})
 def test_writer_precision_nano_fail():
-    """ if writer is not set to nano, supplying this timestamp should be truncated """
+    """if writer is not set to nano, supplying this timestamp should be truncated"""
     ts, pkt = (Decimal('1454725786.010203045'), b'foo')
-    writer.writepkt(pkt, ts=ts)
+    writer.writepkt(pkt, ts=ts)  # noqa
     return [(1454725786.010203, pkt)]
 
+
 @WriterTestWrap()
 def test_writepkt_no_time():
     ts, pkt = 1454725786.526401, b'foooo'
     _tmp = time.time
     time.time = lambda: ts
-    writer.writepkt(pkt)
+    writer.writepkt(pkt)  # noqa
     time.time = _tmp
     return [(ts, pkt)]
 
+
 @WriterTestWrap(writer={'snaplen': 10})
 def test_writepkt_snaplen():
     ts, pkt = 1454725786.526401, b'foooo'
-    writer.writepkt(pkt, ts)
+    writer.writepkt(pkt, ts)  # noqa
     return [(ts, pkt)]
 
+
 @WriterTestWrap()
 def test_writepkt_with_time():
     ts, pkt = 1454725786.526401, b'foooo'
-    writer.writepkt(pkt, ts)
+    writer.writepkt(pkt, ts)  # noqa
     return [(ts, pkt)]
 
+
 @WriterTestWrap()
 def test_writepkt_time():
     ts, pkt = 1454725786.526401, b'foooo'
-    writer.writepkt_time(pkt, ts)
+    writer.writepkt_time(pkt, ts)  # noqa
     return [(ts, pkt)]
 
+
 @WriterTestWrap()
 def test_writepkts():
-    """ writing multiple packets from a list """
+    """writing multiple packets from a list"""
     pkts = [
         (1454725786.526401, b"fooo"),
         (1454725787.526401, b"barr"),
@@ -488,19 +604,5 @@ def test_writepkts():
         (1454725789.526401, b"lol"),
     ]
 
-    writer.writepkts(pkts)
+    writer.writepkts(pkts)  # noqa
     return pkts
-
-if __name__ == '__main__':
-    test_reader()
-    test_pcap_endian()
-    test_writer_precision_normal()
-    test_writer_precision_nano()
-    test_writer_precision_nano_fail()
-    test_writepkt_snaplen()
-    test_writepkt_no_time()
-    test_writepkt_with_time()
-    test_writepkt_time()
-    test_writepkts()
-
-    print('Tests Successful...')
